//-----------------------------------------------------------------------------
// Midi Theremin
// Copyright 2020 Peter Balch
// subject to the GNU General Public License
// reads sonar distances and sends commands to a VS1053 over SPI
//-----------------------------------------------------------------------------

#include <SPI.h>
#include <SimpleST7735.h>
#include <EEPROM.h>

//-----------------------------------------------------------------------------
// Defines and Typedefs
//-----------------------------------------------------------------------------

enum TLeftHandKind {lhVolume, lhTremolo, lhVibrato, lhPitchBendUp, lhPitchBendDown, lhReverb, lhPolyphony, lhChordSize};

//-----------------------------------------------------------------------------
// Global Constants
//-----------------------------------------------------------------------------

const long BAUDRATE  = 9600;  // Baud rate of UART in bps

const byte Version = 2;

const int  SCLK  = 13;
const int  MIDI_DREQ  = A1;
const int  MIDI_RST   = A5;
const int  MIDI_CS    = A0;
const int  BUTTON     = 6;
const int  FOOT_PEDAL = 7;
const int  OLED_CS = 3;
const int  OLED_DC = 2;

const byte UStrigR = 5;
const byte USechoR = 4;
const byte UStrigL = 10;
const byte USechoL = 9;
const int maxDuration = 600;
const int minDuration = 30;
const int BinMin = 0;
const byte maxPolyphony = 7;
const int PitchWheelMax = 128 * 12 * 2; // pitch-wheel max, 128 = 1 semitone
const byte CurVelocity = 127;
const byte NumSetups = 5;

const int MaxScaleNotes = 10;
const PROGMEM byte ScalesArray[] = {
  7, 0, 2, 4, 5, 7, 9, 11, 0, 0,   // C, D, E, F, G, A, B, C              major_Heptatonic
  7, 0, 2, 3, 5, 7, 8, 10, 0, 0,   // C, D, Eb, F, F#, A, Bb, C           minor_Heptatonic
  5, 0, 2, 5, 7, 9, 0, 0, 0, 0,    // C, D, F, G, B, C                    major_Pentatonic
  5, 0, 3, 5, 7, 10, 0, 0, 0, 0,   // C, D#, F, G, B#, C                  minor_Pentatonic
  7, 0, 2, 4, 5, 7, 8, 10, 0, 0,   // C, D, E, F, G, Ab, Bb, C            Aeolian_Dominant_Hindu
  7, 0, 2, 3, 5, 7, 8, 10, 0, 0,   // C, D, Eb, F, G, Ab, Bb, C           Aeolian_Mode
  8, 0, 2, 3, 5, 6, 7, 8, 11, 0,   // C, D, Eb, F, F#, G, Ab, B, C        Algerian
  7, 0, 2, 4, 5, 6, 8, 10, 0, 0,   // C, D, E, F, F#, G#, Bb, C           Arabic
  6, 0, 3, 4, 7, 8, 11, 0, 0, 0,   // C, D#, E, G, Ab, B, C               Augmented
  5, 0, 1, 3, 7, 8, 0, 0, 0, 0,    // C, Db, Eb, G, Ab, C                 Balinese
  8, 0, 2, 4, 5, 7, 8, 9, 11, 0,   // C, D, E, F, G, G#, A, B, C          Bebop
  8, 0, 2, 3, 4, 5, 7, 9, 10, 0,   // C, D, Eb, E, F, G, A, Bb, C         Bebop_Minor
  6, 0, 3, 5, 6, 7, 10, 0, 0, 0,   // C, Eb, F, F#, G, Bb, C              Blues
  7, 0, 1, 4, 5, 7, 8, 11, 0, 0,   // C, Db, E, F, G, Ab, B, C            Byzantine
  5, 0, 4, 6, 7, 11, 0, 0, 0, 0,   // C, E, F#, G, B, C                   Chinese
  8, 0, 2, 3, 5, 6, 8, 9, 11, 0,   // C, D, Eb, F, Gb, Ab, A, B, C        Diminished
  8, 0, 1, 3, 4, 6, 7, 9, 10, 0,   // C, Db, D#, E, F#, G, A, Bb, C       Dominant_Diminished
  7, 0, 2, 3, 5, 7, 9, 10, 0, 0,   // C, D, Eb, F, G, A, Bb, C            Dorian_Mode
  5, 0, 2, 5, 7, 10, 0, 0, 0, 0,   // C, D, F, G, Bb, C                   Egyptian
  8, 0, 1, 3, 4, 5, 6, 8, 10, 0,   // C, C#, D#, E, F, F#, G#, A#, C      Eight_Tone_Spanish
  7, 0, 2, 3, 5, 7, 8, 10, 0, 0,   // C, D, Eb, F, G, Ab, Bb, C           Geez
  5, 0, 1, 5, 6, 10, 0, 0, 0, 0,   // C, Db, F, Gb, Bb, C                 Hirajoshi
  7, 0, 2, 3, 6, 7, 8, 11, 0, 0,   // C, D, Eb, F#, G, Ab, B, C           Hungarian
  5, 0, 1, 5, 7, 10, 0, 0, 0, 0,   // C, C#, F, G, A#, C                  In_sen
  7, 0, 2, 4, 5, 7, 9, 11, 0, 0,   // C, D, E, F, G, A, B, C              Ionian_Mode
  7, 0, 1, 3, 5, 6, 8, 10, 0, 0,   // C, Db, Eb, F, Gb, Ab, Bb, C         Locrian_Mode
  7, 0, 2, 4, 6, 7, 9, 10, 0, 0,   // C, D, E, F#, G, A, Bb, C            Lydian_Dominant
  7, 0, 2, 4, 6, 7, 9, 11, 0, 0,   // C, D, E, F#, G, A, B, C             Lydian_Mode
  7, 0, 1, 4, 5, 7, 8, 11, 0, 0,   // C, Db, E, F, G, Ab, B, C            Maqams
  7, 0, 2, 4, 5, 7, 9, 10, 0, 0,   // C, D, E, F, G, A, Bb, C             Mixolydian_Mod
  7, 0, 1, 3, 5, 7, 8, 11, 0, 0,   // C, Db, Eb, F, G, Ab, B, C           Neapolitan
  8, 0, 2, 3, 4, 6, 7, 8, 9, 11,   // C, D, Eb, E, F#, G, G#, A, B, C     Nine_Tone
  8, 0, 1, 3, 4, 6, 7, 9, 10, 0,   // C, Db, Eb, E, Gb, G, A, Bb, C       Octatonic
  7, 0, 1, 4, 5, 6, 9, 10, 0, 0,   // C, Db, E, F, Gb, A, Bb, C           Oriental
  7, 0, 1, 3, 5, 7, 8, 10, 0, 0,   // C, Db, Eb, F, G, Ab, Bb, C          Phrygian_Mode
  7, 0, 2, 3, 6, 7, 9, 10, 0, 0,   // C, D, Eb, F#, G, A, A#, C           Romanian_Minor
  7, 0, 1, 4, 5, 7, 8, 10, 0, 0,   // C, Db, E, F, G, Ab, Bb, C           Spanish_Gypsy
  7, 0, 1, 3, 4, 6, 8, 10, 0, 0,   // C, Db, D#, E, Gb, G#, Bb, C         Super_Locrian
  6, 0, 2, 4, 6, 8, 10, 0, 0, 0,   // C, D, E, F#, G#, Bb, C              Whole_Tone
  5, 0, 2, 5, 7, 9, 0, 0, 0, 0};   // C, D, F, G, A, C                    Yo

const int MaxChordNotes = 5;
const PROGMEM byte ChordsArray[] = {
  1, 0, 0, 0, 0, // single note
  3, 0, 4, 7  , 0, // Major triad
  4, 0, 4, 7, 9, // Major sixth
  4, 0, 4, 7, 10, // Dominant seventh
  4, 0, 4, 7, 11, // Major seventh
  3, 0, 4, 8  , 0, // Augmented triad
  4, 0, 4, 8, 10, // Augmented seventh
  3, 0, 3, 7  , 0, // Minor triad
  4, 0, 3, 7, 9, // Minor sixth
  4, 0, 3, 7, 10, // Minor seventh
  4, 0, 3, 7, 11, // Minor-major seventh
  3, 0, 3, 6  , 0, // Diminished triad
  4, 0, 3, 6, 9, // Diminished seventh
  4, 0, 3, 6, 10}; // Half-diminished seventh

enum TMenuVariable {
  chMode,                 // 'a' 0=discrete 1=continuous
  chScale,                // 'b' 0..39 (name of scale)
  chOctaves,              // 'c' 1..6
  chLowestnote,           // 'd' 0..127
  chChord,                // 'e' 0..13
  chInversion,            // 'f' 0..3
  chPolyphony,            // 'g' 1..7
  chLeftHandKind,         // 'h' 0..7  (0 LeftHand Volume .. 7 LeftHand ChordSize)
  chMaxDepth,             // 'i' 0..127 (applies to left-hand Vibrato, or Tremolo or PitchBend)
  chTremoloSize,          // 'j' 0..127
  chTremoloPeriod,        // 'k' 0..127
  chVibratoSize,          // 'l' 0..127 (current size of fixed Vibrato; 0x1FFF/24 is one semitone)
  chVibratoPeriod,        // 'm' 0..127
  chContinuousPitchRange, // 'n' 1..48
  chContinuousMiddlenote, // 'o' 0..127
  chInstrument,           // 'p' 0=Acoustic_Grand_Piano .. 127=Gunshot
  chUnused1,              // 'q' unused - available for other commands
  chSwapHands};           // 'r' 0=normal; 1=swap left/right
  
#define sInstrumentName \
      "Grand Piano/" \
      "Bright Piano/" \
      "\bElectric Grand Piano/" \
      "Honkytonk Piano/" \
      "Electric Piano 1/" \
      "Electric Piano 2/" \
      "Harpsichord/" \
      "Clavichord/" \
      "Celesta/" \
      "Glockenspiel/" \
      "Music Box/" \
      "Vibraphone/" \
      "Marimba/" \
      "Xylophone/" \
      "Tubular Bells/" \
      "Dulcimer/" \
      "DrawBar Organ/" \
      "Percussive Organ/" \
      "Rock Organ/" \
      "Church Organ/" \
      "Reed Organ/" \
      "Accordion/" \
      "Harmonica/" \
      "Tango Accordion/" \
      "Guitar nylon/" \
      "Steel Guitar/" \
      "\bElectric Guitar jazz/" \
      "\bElectric Guitar clean/" \
      "\bElectric Guitar muted/" \
      "\bOverdriven Guitar/" \
      "\bDistortion Guitar/" \
      "Guitar Harmonics/" \
      "Acoustic Bass/" \
      "\bElectric Bass finger/" \
      "\bElectric Bass pick/" \
      "Fretless Bass/" \
      "Slap Bass 1/" \
      "Slap Bass 2/" \
      "Synth Bass 1/" \
      "Synth Bass 2/" \
      "Violin/" \
      "Viola/" \
      "Cello/" \
      "Contrabass/" \
      "Tremolo Strings/" \
      "\bPizzicato Strings/" \
      "Orchestral Harp/" \
      "Trimpani/" \
      "\bString Ensembles 1/" \
      "\bString Ensembles 2/" \
      "Synth Strings 1/" \
      "Synth Strings 2/" \
      "Choir Aahs/" \
      "Voice oohs/" \
      "Synth Voice/" \
      "Orchestra Hit/" \
      "Trumpet/" \
      "Trombone/" \
      "Tuba/" \
      "Muted Trumpet/" \
      "French Horn/" \
      "Brass Section/" \
      "Synth Brass 1/" \
      "Synth Brass 2/" \
      "Soprano Sax/" \
      "Alto Sax/" \
      "Tenor Sax/" \
      "Baritone Sax/" \
      "Oboe/" \
      "English Horn/" \
      "Bassoon/" \
      "Clarinet/" \
      "Piccolo/" \
      "Flute/" \
      "Recorder/" \
      "Pan Flute/" \
      "Blown Bottle/" \
      "Shakuhachi/" \
      "Whistle/" \
      "Ocarina/" \
      "Square Lead/" \
      "Saw Lead/" \
      "Calliope/" \
      "Chiff Lead/" \
      "Charang Lead/" \
      "Voice Lead/" \
      "Fifths Lead/" \
      "Bass Lead/" \
      "New Age/" \
      "Warm Pad/" \
      "Polysynth Pad/" \
      "Choir Pad/" \
      "Bowed Pad/" \
      "Metallic Pad/" \
      "Halo Pad/" \
      "Sweep Pad/" \
      "Rain FX/" \
      "Sound Track FX/" \
      "Crystal FX/" \
      "Atmosphere FX/" \
      "Brigthness FX/" \
      "Goblins FX/" \
      "Echoes FX/" \
      "SciFi/" \
      "Sitar/" \
      "Banjo/" \
      "Shamisen/" \
      "Koto/" \
      "Kalimba/" \
      "Bagpipe/" \
      "Fiddle/" \
      "Shanai/" \
      "Tinkle Bell/" \
      "Agogo/" \
      "\bPitched Percussion/" \
      "Woodblock/" \
      "Taiko/" \
      "Melodic Tom/" \
      "Synth Drum/" \
      "Reverse Cymbal/" \
      "\bGuitar Fret Noise/" \
      "Breath Noise/" \
      "Seashore/" \
      "Bird/" \
      "Telephone/" \
      "Helicopter/" \
      "Applause/" \
      "Gunshot"
static const char aInstrumentName[] PROGMEM = sInstrumentName;

#define sLeftHandName "Volume/Tremolo/Vibrato/PitchBendUp/PitchBendDown/Reverb/Polyphony/ChordSize"
static const char aLeftHandName[] PROGMEM = sLeftHandName;

const byte BigDigitsFont[] PROGMEM = {
  16, // height including descender
  0, // descender
  '1', // first char
  7, 0x00,0x10,0x00,0x30,0x00,0x30,0xFF,0x7F,0xFF,0x7F,0xFF,0xFF,0xFF,0xFF,  // 1
  10, 0x03,0x30,0x07,0x60,0x0F,0xE0,0x1B,0xE0,0x73,0xF0,0xE3,0xFF,0xC3,0x7F,0x03,0x7F,0x03,0x1C,0x0F,0x00,  // 2
  11, 0x02,0x00,0x07,0x10,0x03,0x20,0x01,0x40,0x01,0xC1,0x81,0xE3,0xC3,0xFF,0xFF,0xFF,0xFF,0x7D,0xFE,0x30,0x38,0x00,  // 3
  11, 0x78,0x00,0xB8,0x01,0x38,0x02,0x38,0x04,0x38,0x08,0xFF,0x1F,0xFF,0x7F,0xFF,0xFF,0xFF,0xFF,0x38,0x00,0x38,0x00,  // 4
  11, 0x02,0x03,0x07,0x0F,0x07,0x37,0x03,0xE7,0x01,0xE7,0x81,0xE7,0x81,0xE3,0xC3,0xE3,0xE7,0xE3,0xFE,0xE1,0xF8,0x80,  // 5
  0};

  
//-----------------------------------------------------------------------------
// Global Variables
//-----------------------------------------------------------------------------

byte curBin = 255;

//number of notes in scale then offsets in semitones from start of scale
signed char Scale[MaxScaleNotes] = {7, 0, 2, 4, 5, 7, 9, 11, 0}; // C, D, E, F, G, A, B, C           major_Heptatonic

//number of notes in Chord then offsets in semitones from start of Chord
signed char Chord[MaxChordNotes] = {1, 0, 0, 0, 0};  // single note; Chord[0] says how many notes in chord
byte ChordMax = 255; // max number of notes in chord - if conrolled by left hand

int PitchBend = 0; // range -8191..8192; current size of pitch bend from either right or left hand; 0x2000/24 is one semitone
long ValueLH = 0; // left-hand sonar value; in range 0..1023

byte CurSetupNumber = 1;

//-----------------------------------------------------------------------------
// Global Variables for Menu
//-----------------------------------------------------------------------------

int MenuCur0 = 0;
int MenuCur1 = 0;
bool IsMenuValue = false;
byte MenuCurVar;
bool IsMenuIncrementing = false;
bool IsMenuing = false;

byte MenuVariable[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
byte MenuVariableMin[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
byte MenuVariableMax[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

byte PrevSetup = 1;

//-----------------------------------------------------------------------------
//  SPItransferMidi
//    send a byte to the midi chip
//-----------------------------------------------------------------------------
void SPItransferMidi(byte b)
{                  
  while (digitalRead(MIDI_DREQ) == 0 ) ;

  digitalWrite(MIDI_CS, LOW);
  SPI.transfer(b);
  SPI.transfer(0x00); // to read result
  digitalWrite(MIDI_CS, HIGH);
  delayMicroseconds(100);
}

//-----------------------------------------------------------------------------
//  SPItransferMidi3
//    sends 3 bytes to the midi chip
//-----------------------------------------------------------------------------
void SPItransferMidi3(byte a, byte b, byte c)
{
  if ((b < 128) && (c < 128)) {
    SPItransferMidi(a);
    SPItransferMidi(b);
    SPItransferMidi(c);
  }
}

//-----------------------------------------------------------------------------
//  GetSerial
//    waits for and receives a serial byte
//-----------------------------------------------------------------------------
byte GetSerial() {
  byte b;
  while ( Serial.available() == 0 ) ;
  b = Serial.read();
  return b;
}

//-----------------------------------------------------------------------------
//  SaveToEEPROM
//    save setup to EEPROM
//    SetupNum is from 1 to NumSetups
//-----------------------------------------------------------------------------
void SaveToEEPROM(byte SetupNum) {
  byte i;
  EEPROM.update(0, Version);
  for (i=0; i<sizeof(MenuVariable); i++) 
    EEPROM.update(i+1+sizeof(MenuVariable)*(SetupNum-1), MenuVariable[i]);
}

//-----------------------------------------------------------------------------
//  LoadFromEEPROM
//    Load setup from EEPROM
//    SetupNum is from 1 to NumSetups
//-----------------------------------------------------------------------------
void LoadFromEEPROM(byte SetupNum) {
  byte i;

  PlayNote(255, false); 
  if (EEPROM.read(0) == Version) { 
    for (i=0; i<sizeof(MenuVariable); i++) 
      MenuVariable[i] = EEPROM.read(i+1+sizeof(MenuVariable)*(SetupNum-1));
    SetScale();
    SetChord();
    ChordMax = 255;
    SPItransferMidi(0xC0); SPItransferMidi(MenuVariable[chInstrument]); 
    SetLeftHandKind(); 
    CurSetupNumber = SetupNum;
  }
  PrevSetup = SetupNum;
}

//-----------------------------------------------------------------------------
//  GetSonar
//    measure a sonar sensor
//-----------------------------------------------------------------------------
long GetSonar(byte UStrig, byte USecho) {
  long duration;

  do {
    delay(10);

    digitalWrite(UStrig, HIGH);
    delayMicroseconds(10);
    digitalWrite(UStrig, LOW);
    //  delayMicroseconds(1000);

    while (digitalRead(USecho) == LOW) ;
    duration = 0;
    while ((digitalRead(USecho) == HIGH) && (duration < maxDuration))
      duration++;
  } while (duration < 10);

  return duration;
}

//-----------------------------------------------------------------------------
//  BinToNote
//    converts a sonar bin to a note number
//-----------------------------------------------------------------------------
byte BinToNote(byte bin) {
  int BinMax;
  int numScale;

  numScale = Scale[0];

  BinMax = numScale * MenuVariable[chOctaves];
  bin = BinMax - bin;

  return MenuVariable[chLowestnote] + Scale[bin % numScale + 1] + 12 * (bin / numScale);
}

//-----------------------------------------------------------------------------
//  DurationToBin
//    converts a sonar duration to a bin number
//-----------------------------------------------------------------------------
byte DurationToBin(long duration) {
  int c, binWidth;
  int BinMax;

  if (duration >= maxDuration)
    return 255;
  if (duration <= minDuration)
    return 255;

  BinMax = Scale[0] * (int)MenuVariable[chOctaves];
  binWidth = ((maxDuration - minDuration) * (4 - 1) / 4 / (BinMax - BinMin));
  c = ((curBin - BinMin) * (maxDuration - minDuration) + (maxDuration - minDuration) / 2) / (BinMax - BinMin + 1) + minDuration;
  if ((duration < c + binWidth) && (duration > c - binWidth))
    return curBin;

  return map(duration, minDuration, maxDuration, BinMin, BinMax+1);
}

//-----------------------------------------------------------------------------
//  StartChord
//    starts playing several notes in a chord
//-----------------------------------------------------------------------------
void StartChord(byte note) {
  byte i;
  for (i = 1; i <= Chord[0] && i <= ChordMax; i++)
    SPItransferMidi3(0x90, note + Chord[i], CurVelocity); // start note

  if (!IsMenuing)
    DrawKeyboard(note);
}

//-----------------------------------------------------------------------------
//  StopChord
//    staps playing the in a chord
//-----------------------------------------------------------------------------
void StopChord(byte note) {
  byte i;
  for (i = 1; i <= Chord[0]; i++)
    SPItransferMidi3(0x80, note + Chord[i], CurVelocity); // stop note
    
  if (!IsMenuing)
    DrawKeyboard(255);
}

//-----------------------------------------------------------------------------
//  PlayNote
//    start playing a single note
//    if note = 255 then stop all notes
//    if note > 200 then set polyphony to note-200
//    else start/stop note
//-----------------------------------------------------------------------------
void PlayNote(byte note, bool start) {
  static byte LiveNote[maxPolyphony] = {255};
  byte i, j;

  if (note <= 127) {
    if (start) { // start a note
      if (LiveNote[MenuVariable[chPolyphony] - 1] <= 127)
      {
        StopChord(LiveNote[MenuVariable[chPolyphony] - 1]); // stop note
      }
      LiveNote[MenuVariable[chPolyphony] - 1] = 255;

      for (j = maxPolyphony - 1; j > 0; j--)
        LiveNote[j] = LiveNote[j - 1]; // move all up by 1 place
      LiveNote[0] = note;
      StartChord(note); // start note
    } else { // stop a note
      StopChord(note); // stop note
      for (i = 0; i < maxPolyphony; i++) {
        if (LiveNote[i] == note) {
          for (j = i; j < maxPolyphony - 1; j++)
            LiveNote[j] = LiveNote[j + 1]; // move all above down by one
          LiveNote[maxPolyphony - 1] = 255;
        }
      }
    }
  } else if (note == 255) { // stop all notes
    for (i = 0; i < maxPolyphony; i++)
      if (LiveNote[i] <= 127) {
        StopChord(LiveNote[i]); // stop note
        LiveNote[i] = 255;
      }
  } else { // set polyphony
    MenuVariable[chPolyphony] = constrain(MenuVariable[chPolyphony], 1, maxPolyphony);
    for (i = MenuVariable[chPolyphony]; i < maxPolyphony; i++) {
      StopChord(LiveNote[i]); // stop note
      LiveNote[i] = 255;
    }
  }
}

//-----------------------------------------------------------------------------
//  MedianOfSeven
//    median filter
//-----------------------------------------------------------------------------
long MedianOfSeven(long a0, long a1, long a2, long a3, long a4, long a5, long a6) {
  long tmp;
  if (a4 < a0) { tmp = a4; a4 = a0; a0 = tmp; }
  if (a5 < a1) { tmp = a5; a5 = a1; a1 = tmp; }
  if (a6 < a2) { tmp = a6; a6 = a2; a2 = tmp; }
  if (a2 < a0) { tmp = a2; a2 = a0; a0 = tmp; }
  if (a3 < a1) { tmp = a3; a3 = a1; a1 = tmp; }
  if (a6 < a4) { tmp = a6; a6 = a4; a4 = tmp; }
  if (a4 < a2) { tmp = a4; a4 = a2; a2 = tmp; }
  if (a5 < a3) { tmp = a5; a5 = a3; a3 = tmp; }
  if (a1 < a0) { tmp = a1; a1 = a0; a0 = tmp; }
  if (a3 < a2) { tmp = a3; a3 = a2; a2 = tmp; }
  if (a5 < a4) { tmp = a5; a5 = a4; a4 = tmp; }
  if (a4 < a1) { tmp = a4; a4 = a1; a1 = tmp; }
  if (a6 < a3) { tmp = a6; a6 = a3; a3 = tmp; }
  if (a4 < a3) return a4; else return a3;
}

//-----------------------------------------------------------------------------
//  GetBin
//    convert a sonar duration into a bin number
//    filter duration by ignoring maxDuration until there hace been 10 of them
//    then do Median Of Seven
//-----------------------------------------------------------------------------
byte GetBin(long duration) {
  byte bin;
  static int numMaxes = 100;
  static long prevDuration = maxDuration;
  static long d1 = maxDuration;
  static long d2 = maxDuration;
  static long d3 = maxDuration;
  static long d4 = maxDuration;
  static long d5 = maxDuration;
  static long d6 = maxDuration;

  // filter out occasional loss of echo
  if (duration >= maxDuration) {
    numMaxes++;
    if (numMaxes < 10)
      duration = prevDuration;
  } else {
    numMaxes = 0;
  }
  prevDuration = duration;

  // convert to bin number
  bin = DurationToBin(MedianOfSeven(d1, d2, d3, d4, d5, d6, duration));
  d6 = d5;  d5 = d4;  d4 = d3;  d3 = d2;  d2 = d1;  d1 = duration;

  return bin;
}

//-----------------------------------------------------------------------------
//  SetChord
//    sets the Chord array to a chord note set
//-----------------------------------------------------------------------------
void SetChord() {
  byte i;
  for (i = 0; i < MaxChordNotes; i++)
    Chord[i] = pgm_read_byte_near(ChordsArray + (int)MenuVariable[chChord] * MaxChordNotes + i);

  switch (MenuVariable[chInversion]) {
    case 1: Chord[2] -= 12; //  first inversion:  notes 2,3,4 down one octave
    case 2: Chord[3] -= 12; //  second inversion: notes 3,4 down one octave
    case 3: Chord[4] -= 12; //  third inversion:  notes 4 down one octave
  }
}

//-----------------------------------------------------------------------------
//  SetScale
//    sets the Scale array to a scale note set
//-----------------------------------------------------------------------------
void SetScale() {
  byte i;
  for (i = 0; i < MaxScaleNotes; i++)
    Scale[i] = pgm_read_byte_near(ScalesArray + (int)MenuVariable[chScale] * MaxScaleNotes + i);
}

//-----------------------------------------------------------------------------
//  SetLeftHandKind
//    sets the function of the left hand
//-----------------------------------------------------------------------------
void SetLeftHandKind() {
  // set Volume max
  SetVolume(127);
  // turn off LH Vibrato and Pitch Bend
  SetBendValue(0x2000);
  PitchBend = 0;
  ValueLH = 0;
  // turn off SetReverb
  SetReverb(0);
  // Set polyphony to 1
  MenuVariable[chPolyphony] = 1;
  PlayNote(200, false);

  // allow all notes in a chord
  ChordMax = 255;  

  switch (MenuVariable[chLeftHandKind]) {
    case lhVolume: SetVolume(0); break;
    case lhReverb: SPItransferMidi3(0xB0, 0x0C, 127); break;
    case lhChordSize: ChordMax = 1; break;
  }
}

//-----------------------------------------------------------------------------
//  ExecSerialDebug
//    executes a command from the serial port
//-----------------------------------------------------------------------------
void ExecSerialDebug()
{
  long duration;
  byte b,c;

  b = GetSerial();
  c = GetSerial();
  switch (b) {
    case 0:
      duration = GetSonar(UStrigR, USechoR);
      if (duration < maxDuration)
        SPItransferMidi3(0x90, (Scale[0] * (int)MenuVariable[chOctaves] - BinMin + 1)*duration / maxDuration + BinMin, c);
      break;
    case 1:  MenuVariable[chPolyphony] = c; PlayNote(200, false); break;
    case 2:  MenuVariable[chOctaves] = c; break;
    case 3:  MenuVariable[chMaxDepth] = c; break; // c semitones
    case 4:  MenuVariable[chLowestnote] = c; break;
    case 5:  MenuVariable[chContinuousMiddlenote] = c; break;
    case 6:  MenuVariable[chContinuousPitchRange] = c; break; // c semitones
    case 8:  MenuVariable[chMode] = c != 0; break;
    case 9:  MenuVariable[chScale] = c; SetScale(); break;
    case 10: MenuVariable[chLeftHandKind] = c; SetLeftHandKind(); break;
    case 11: MenuVariable[chChord] = c; SetChord(); break;
    case 12: MenuVariable[chVibratoSize] = c; break;
    case 13: MenuVariable[chVibratoPeriod] = c; break; // 100 = once per sec
    case 14: MenuVariable[chTremoloSize] = c; break;
    case 15: MenuVariable[chTremoloPeriod] = c; break; // 100 = approx once per sec
    case 16: break; // not used
    case 17: SPItransferMidi(0xC0); SPItransferMidi(c); break;
  }
}

//-----------------------------------------------------------------------------
//  SetBendRange
//    sets the bend range
//-----------------------------------------------------------------------------
void SetBendRange(int range)
{
  SPItransferMidi3(0xB0, 0x65, 0x00);
  SPItransferMidi3(0xB0, 0x64, 0x00);
  SPItransferMidi3(0xB0, 0x06, range / 128);
  SPItransferMidi3(0xB0, 0x26, range % 128);
  SPItransferMidi3(0xB0, 0x65, 0x7F);
  SPItransferMidi3(0xB0, 0x64, 0x7F);
}

//-----------------------------------------------------------------------------
//  SetBendValue
//    sets the bend value
//-----------------------------------------------------------------------------
void SetBendValue(int value)
{
  value = constrain(value, 0, 0x3FFF);
  SPItransferMidi3(0xE0, value % 128, value / 128);
}

//-----------------------------------------------------------------------------
//  SetVolume
//    sets the volume
//-----------------------------------------------------------------------------
void SetVolume(int value)
{
  SPItransferMidi3(0xB0, 0x07, constrain(value,0,127));
}

//-----------------------------------------------------------------------------
//  SetReverb
//    sets the reverb size
//-----------------------------------------------------------------------------
void SetReverb(int value)
{
  SPItransferMidi3(0xB0, 0x5B, constrain(value,0,127));
}

//-----------------------------------------------------------------------------
//  ApplyTremolo
//    applies Tremolo to the current channel
//-----------------------------------------------------------------------------
void ApplyTremolo(void)
{
  unsigned long time,period;
  byte vol;
  static byte prevVol = 0;

  if ((MenuVariable[chLeftHandKind] == lhTremolo) && (IsMenuing))
    return;
  if (MenuVariable[chLeftHandKind] == lhVolume)
    return;

  if ((MenuVariable[chLeftHandKind] == lhTremolo) || (MenuVariable[chTremoloSize] > 0) || (prevVol > 0)) {
    period = (int)MenuVariable[chTremoloPeriod]*10;
    time = millis() % period;
    if (time > period / 2)
      time = period - time;

    if (MenuVariable[chLeftHandKind] == lhTremolo)
      vol = 127 - time * (long)MenuVariable[chMaxDepth]*ValueLH*20/1023 / period; else
      vol = 127 - time * (long)MenuVariable[chTremoloSize]*20 / period;
  }

  if ((vol > 0) || (prevVol > 0))
    SetVolume(vol);
  prevVol = vol;
}

//-----------------------------------------------------------------------------
//  CalcVibratoPitchBend
//    applies pitch bend to the current channel
//-----------------------------------------------------------------------------
int CalcVibratoPitchBend(void)
{
  long time,period;

  period = (int)MenuVariable[chVibratoPeriod]*10;
  time = millis() % period;

  if (time > period / 2)
    time = period - time;
  time = time - period / 4;

  if (MenuVariable[chLeftHandKind] == lhVibrato) {
    if (IsMenuing)
      return 0;
    return time * ValueLH * (long)MenuVariable[chMaxDepth] / 5 / period + PitchBend; 
  } else
    return time * ((long)MenuVariable[chVibratoSize] * 0x2000) / 24 / 4 * 4 / period + PitchBend;
}

//-----------------------------------------------------------------------------
//  ExecContinuous
//    executes a right-hand sonar duration to play notes in continuous mode
//-----------------------------------------------------------------------------
void ExecContinuous(long duration) {
  static int numMaxes = 100;
  static int prevNearestNote = 0;
  static bool PlayingNote = false;
  static long prevValue = 0;
  const int nSmooth = 5;
  int Value,nearestNote,Range;

  if (duration >= maxDuration) {
    numMaxes++;  // filter out occasional loss of echo
    if (numMaxes < 10)
      return;
    numMaxes = 100;

    if (PlayingNote) {
      PlayNote(255, false);
      if (!IsMenuing)
        DrawKeyboard(255);
    }
    PlayingNote = false;
  } else {
    numMaxes = 0;

    Range = constrain(((long)MenuVariable[chContinuousPitchRange] * 0x2000) / 24, 0, 0x1FFF); // Contin Pitch Range semitones
    Value = map(duration, minDuration, maxDuration, +Range/2, -Range/2);

    if (!PlayingNote) {
      prevValue = Value;
      PitchBend = prevValue;
      SetBendValue(0x2000 + CalcVibratoPitchBend());
      PlayNote(MenuVariable[chContinuousMiddlenote], true);
      PlayingNote = true;
    } else {
      prevValue = (Value + prevValue * (nSmooth - 1)) / nSmooth;
      PitchBend = prevValue;
      SetBendValue(0x2000 + CalcVibratoPitchBend());
    }

    if (!IsMenuing) {
      nearestNote = map(Value, -Range/2, +Range/2, 
        10*((int)MenuVariable[chContinuousMiddlenote]+MenuVariable[chContinuousPitchRange]/2),
        10*((int)MenuVariable[chContinuousMiddlenote]-MenuVariable[chContinuousPitchRange]/2));

      if (abs(nearestNote - prevNearestNote) > 15) {
        DrawKeyboard(nearestNote / 10);
        prevNearestNote = nearestNote;
      }
    }
  }
}

//-----------------------------------------------------------------------------
//  ExecDiscrete
//    executes a right-hand sonar duration to play notes in discrete mode
//-----------------------------------------------------------------------------
void ExecDiscrete(long duration)
{
  byte bin;

  bin = GetBin(duration);

  if (bin == 255) 
    PlayNote(255, false); // stop all

  // play note if bin number changed
  if (bin != curBin) {
    curBin = bin;
    if (bin <= 127) {
      PlayNote(BinToNote(bin), true);
    }
  }

  switch (MenuVariable[chLeftHandKind]) {
    case lhVibrato:
    case lhPitchBendUp:
    case lhPitchBendDown: if (!IsMenuing) SetBendValue(0x2000 + CalcVibratoPitchBend()); break;
    default: if (MenuVariable[chVibratoSize] > 0) SetBendValue(0x2000 + CalcVibratoPitchBend());
  }
}

//-----------------------------------------------------------------------------
//  Beep
//    make a Beep sound (for menu)
//    n is note number
//    n == 255 means silence it (because it sometimes gets stuck on)
//-----------------------------------------------------------------------------
void Beep(byte n)
{
  if (n == 0xFF) {
    SPItransferMidi3(0xB1, 0x78, 0x00);
  } else {
    SPItransferMidi3(0x91, n, 127); // start note
    delay(100);
    SPItransferMidi3(0x81, n, 127); // stop note
  }
}

//-----------------------------------------------------------------------------
//  ExecLeftHand
//    executes a left-hand sonar duration to adjust notes
//-----------------------------------------------------------------------------
void ExecLeftHand(long duration)
{
  static int numMaxes = 100;
  const int nSmooth = 5;
  static bool CanSeeHand = false;
  int Value,i;
  const int menuDist = 70;
  static unsigned long NextMenuRedraw = 0;
  static byte count = 0;

  if (IsMenuing) {  
    count++;  
    if ((count % 2 > 0)  || (millis() < NextMenuRedraw)) 
      return;
  }

  if (duration >= maxDuration) {
    numMaxes++;  // filter out occasional loss of echo
    if (numMaxes < 10)
      return;
    numMaxes = 100;
    Value = 0;
    CanSeeHand = false;
  } else {
    numMaxes = 0;
    Value = map(duration, maxDuration, minDuration, 0, 1023);
    if (!CanSeeHand)
      ValueLH = Value;
    CanSeeHand = true;
  }

  if (IsMenuing) {    
    switch ((1023-Value)/menuDist) {
      case 0 ... 2: Beep(90); MenuPlus(); NextMenuRedraw = 0; Beep(255); break;
      case 3 ... 6: Beep(93); MenuMinus(); NextMenuRedraw = 0; Beep(255); break;
    }
  } else {
    if (Value != ValueLH) {
      ValueLH = (Value + ValueLH * (nSmooth - 1)) / nSmooth;

      switch (MenuVariable[chLeftHandKind]) {
        case lhVolume:        SetVolume(ValueLH / 8); break;
        case lhPitchBendUp:   PitchBend = +ValueLH * (long)MenuVariable[chMaxDepth] / 20; break;
        case lhPitchBendDown: PitchBend = -ValueLH * (long)MenuVariable[chMaxDepth] / 20; break;
        case lhReverb:        SetReverb(ValueLH / 8); break;
        case lhPolyphony:     MenuVariable[chPolyphony] = ValueLH / 160; PlayNote(200, false); break;
        case lhChordSize:     ChordMax = ValueLH * Chord[0] / 1023 + 1; break;
      }
    }
  }
}

bool NextCharIs(word *s, char c)
{
  if (pgm_read_byte_near(*s) == c)
  {
    (*s)++;
    return true;
  } else
    return false;
}

char NextChar(word *s)
{
  char c = pgm_read_byte_near(*s);
  (*s)++;
  return c;
}

int NextNumber(word *s)
{
  int result = 0;
  char c = pgm_read_byte_near(*s);
  while ((c >= '0') and (c <= '9'))
  {
    result = result * 10 + c - '0';
    (*s)++;
    c = pgm_read_byte_near(*s);
  }
  return result;
}

//-----------------------------------------------------------------------------
//  AppendCharToString
//    used in menuing
//    appends a chart to a string
//-----------------------------------------------------------------------------
void AppendCharToString(char *s, char c, int maxlen) {
  int k = strlen(s);
  if (k < maxlen - 1) {
    s[k] = c;
    s[k + 1] = 0;
  }
}

//-----------------------------------------------------------------------------
//  AppendIntToString
//    used in menuing
//    appends an int value to a string
//-----------------------------------------------------------------------------
void AppendIntToString(char *s, int i, int maxlen) {
  if (i < 0) {
    i = -i;
    AppendCharToString(s, '-', maxlen);
  }

  bool HasDigit = false;
  int n =  10000;
  if (i == 0) {
    AppendCharToString(s, '0', maxlen);
  } else {
    while (n > 0) {
      if ((i >= n) or HasDigit) {
        AppendCharToString(s, '0' + (i / n), maxlen);
        HasDigit = true;
      }
      i %= n;
      n /= 10;
    }
  }
}

//-----------------------------------------------------------------------------
//  DrawMenu
//    draws the menu on the screen
//      <x=1-10> means menu variable 'x' in the range 1..10 (<z> has a special meaning)
//      [x=one/two/three] means a menu variable 'x' in the range 0..2 displayed as an enumerated type
//      ?x=3 means display the rest of the line only if (a = 3)
//      \n means end of line
//      \a means medium font
//      \b means small font (a line always starts in medium font)
//      : are displayed as blank but separate the left-hand / right-hand part of the line
//      #x means display variable 'x' as a note name
//    if current item is Scale then draws scale
//-----------------------------------------------------------------------------
void DrawMenu(bool MovingDown) {
#define DRAWCHAR(c)  if (c == '\b') font = SmallFont; else if (c == '\a') font = MediumFont; else if (RedrawCount == 3) DrawChar(c,font);

  static const char sMenu[] PROGMEM =
    "Instrument\n"
      "  <p=0-127>:[p=" sInstrumentName "]\n"
      "  Swap Hands:[r=normal/swapped]\n"
    "Right Hand\n"
    "  Mode:[a=discrete/continuous]\n"
    "?a=0Left Hand\n"
    "?a=0  Mode:[h=" sLeftHandName "]\n"
    "?a=0?h=1  Max Depth:<i=0-127>\n"
    "?a=0?h=2  Max Depth:<i=0-127>\n"
    "?a=0?h=3  Max Depth:<i=0-127>\n"
    "?a=0?h=4  Max Depth:<i=0-127>\n"
    "?a=0Scale\n"
      "?a=0  Scale:[b="
      "\bmajor Heptatonic/"
      "\bminor Heptatonic/"
      "\bmajor Pentatonic/"
      "\bminor Pentatonic/"
      "\bAeolian Dominant Hindu/"
      "Aeolian Mode/"
      "Algerian/"
      "Arabic/"
      "Augmented/"
      "Balinese/"
      "Bebop/"
      "Bebop Minor/"
      "Blues/"
      "Byzantine/"
      "Chinese/"
      "Diminished/"
      "\bDominant Diminished/"
      "Dorian Mode/"
      "Egyptian/"
      "\bEight Tone Spanish/"
      "Geez/"
      "Hirajoshi/"
      "Hungarian/"
      "In sen/"
      "Ionian Mode/"
      "Locrian Mode/"
      "Lydian Dominant/"
      "Lydian Mode/"
      "Maqams/"
      "Mixolydian Mod/"
      "Neapolitan/"
      "Nine Tone/"
      "Octatonic/"
      "Oriental/"
      "Phrygian Mode/"
      "Romanian Minor/"
      "Spanish Gypsy/"
      "Super Locrian/"
      "Whole Tone/"
      "Yo]\n"
    "?a=0  Octaves:<c=1-6>\n"
    "?a=0  Lowest note:<d=0-127> #d\n"
    "?a=0Chord\n"
    "?a=0  Chord:[e=single note/Major triad/Major sixth/Dominant seventh/Major seventh/Augmented triad/Augmented seventh/Minor triad/Minor sixth/Minor seventh/Minor major seventh/Diminished triad/Diminished seventh/Half diminished seventh]\n"
    "?a=0?e!=0  Inversion:<f=0-3>\n"
    "?a=0  Polyphony:<g=1-7>\n"
    "?a=1Range\n"
    "?a=1  Num semitones:<n=1-48>\n"
    "?a=1  Middle note:<o=0-127> #o\n"
    "?a=1LeftHand\n"
    "?a=1  Mode:[h=Volume/Tremolo/Vibrato]\n"
    "Tremolo\n"
    "?a=0?h!=1  Size:<j=0-127>\n"
    "?a=1  Size:<j=0-127>\n"
    "  Period:<k=1-127>\n"
    "Vibrato\n"
    "?a=0?h!=2  Size:<l=0-127>\n"
    "?a=1  Size:<l=0-127>\n"
    "  Period:<m=1-127>\n";

  static char *noteName[] = {"C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B"};

  char t[50];
  int j, Line0, Line1;
  char c, v ;
  bool Ignore;
  word s;
  byte SubMenuCount = 0;
  byte MainMenuCount = 0;
  int yText = 0;
  word font;
  byte RedrawCount;

  for (RedrawCount=1; RedrawCount<=3; RedrawCount++) {
    if (RedrawCount == 3) ST7735ClearDisplay(ST7735_BLACK);
  
    s = sMenu;
    yText = 0;
    Line0 = -1;
    Line1 = -1;
    v = 255;
    MenuCurVar = 255;
  
    while (pgm_read_byte_near(s) != 0) {
      t[0] = 0;
      Ignore = false;
      while (pgm_read_byte_near(s) != '\n') {
        c = NextChar(&s);
        switch (c) {
          case '[':
            if (!Ignore) {
              v = NextChar(&s) - 'a';
              MenuVariableMin[v] = 0;
              MenuVariableMax[v] = -1;
              j = MenuVariable[v];
              if (NextCharIs(&s, '='))
              {
                do {
                  MenuVariableMax[v]++;
                  while ((pgm_read_byte_near(s) != '/') && (pgm_read_byte_near(s) != ']')) {
                    if (j == 0)
                      AppendCharToString(t, pgm_read_byte_near(s), sizeof(t));
                    s++;
                  }
                  j--;
                } while (NextCharIs(&s, '/'));
              }
              NextCharIs(&s, ']');
            }
            break;
          case '<':
            if (!Ignore) {
              v = NextChar(&s) - 'a';
              if (NextCharIs(&s, '='))
              {
                j = MenuVariable[v];
                AppendIntToString(t, j, sizeof(t));
                MenuVariableMin[v] = NextNumber(&s);
                NextCharIs(&s, '-');
                MenuVariableMax[v] = NextNumber(&s);
              }
              NextCharIs(&s, '>');
            }
            break;
          case '?': {
              NextCharIs(&s, '?');
              j = MenuVariable[NextChar(&s) - 'a'];
              if (NextCharIs(&s, '!'))
              {
                NextCharIs(&s, '=');
                if (j == NextNumber(&s))
                  Ignore = true;
              } else
              {
                NextCharIs(&s, '=');
                if (j != NextNumber(&s))
                  Ignore = true;
              }
            }
            break;
          case '#': {
              NextCharIs(&s, '#');
              j = MenuVariable[NextChar(&s) - 'a'];
              strlcat(t, noteName[j % 12], sizeof(t));
              AppendIntToString(t, (j + 3) / 12 - 1, sizeof(t));
            }
            break;
          default:
            AppendCharToString(t, c, sizeof(t));
        }
      }
  
      if (Ignore) {
        while (pgm_read_byte_near(s) != '\n')
          c = NextChar(&s);
      }
  
      s++;
  
      if (!Ignore) {
        if (t[0] == ' ')
        {
          Line1++;
          if (Line0 != MenuCur0)
            Ignore = true;
        } else {
          Line0++;
          Line1 = -1;
        }
      }
  
      if (!Ignore) {
        char *pt = t;
        ST7735SetCursor(4, yText * 11 + 10 + 4);
  
        if (*pt == ' ')
        {
          font = MediumFont;
          SubMenuCount++;
  
          if ((*pt == ' ') and (MenuCur1 == Line1))
          {
            while (*pt == ' ') {
              DRAWCHAR(' ');
              pt++;
            }
  
            if (IsMenuValue)
              ST7735CurColor = ST7735_YELLOW; else
              ST7735CurColor = ST7735_WHITE;
  
            while ((*pt != ':') && (*pt != 0)) {
              DRAWCHAR(*pt);
              pt++;
            }
  
            DRAWCHAR(' ');
  
            if (*pt == ':') {
              pt++;
              if (IsMenuValue)
                ST7735CurColor = ST7735_WHITE; else
                ST7735CurColor = ST7735_YELLOW;
              while (*pt != 0) {
                DRAWCHAR(*pt);
                pt++;
              }
            }
  
            MenuCurVar = v;
          } else {
            ST7735CurColor = ST7735_YELLOW;
            while (*pt != 0) {
              if (*pt == ':') {
                DRAWCHAR(' ');
              } else {
                DRAWCHAR(*pt);
              }
              pt++;
            }
          }
        } else {
          MainMenuCount++;
          if (Line0 == MenuCur0)
            ST7735CurColor = ST7735_WHITE; else
            ST7735CurColor = ST7735_GREEN;
          while ((*pt != ':') && (*pt != 0)) {
            if (RedrawCount == 3) DrawChar(*pt, MediumBoldFont); 
            pt++;
          }
        }
        yText++;
      }
    }

    if (MenuCurVar == 255) {
      MenuCur1 = min(MenuCur1, SubMenuCount - 1);
      if (MenuCur0 >= MainMenuCount - 1) {
        // gone beyond last menu item
        MenuCur0 = MainMenuCount - 1;
      } else 
      if (MovingDown) {
        // gone down beyond last sub-menu item
        MenuCur0++;
        MenuCur1 = 0;
      } else {
        // gone up above first sub-menu item
      }      
    }      
    if (RedrawCount == 3) 
      switch (MenuCurVar) {
        case chScale:                 
        case chLowestnote:            
          DrawScale();
          break;
        case chChord:                 
        case chInversion:             
          DrawChord();
          break;
      }      
  }
}

//-----------------------------------------------------------------------------
//  DrawPlaying
//    draws the "piano" on the screen
//-----------------------------------------------------------------------------
void DrawPlaying() {
  int i;
    
  ST7735ClearDisplay(ST7735_BLACK);
  DrawKeyboard(254);
  
  ST7735CurColor = ST7735_YELLOW; 
  DrawEnumerated(4,70,MenuVariable[chInstrument], aInstrumentName, MediumBoldFont); 
  ST7735CurColor = ST7735_GREEN; 
  DrawEnumerated(4,86,MenuVariable[chLeftHandKind], aLeftHandName, MediumBoldFont); 

  ST7735CurColor = ST7735_WHITE;
  DrawStringAt(0, 120, "Setup ", MediumBoldFont); 
  for (i=1; i <= NumSetups; i++) {
    ST7735CurColor = ST7735_WHITE;
    if (i == CurSetupNumber) {
      DrawBox(20+i*18,108,18,20);
      ST7735CurColor = ST7735_BLACK;
    }
    ST7735SetCursor(23+i*18,125);
    DrawInt(i, BigDigitsFont);    
  }
}

//-----------------------------------------------------------------------------
//  DrawEnumerated
//    draws the string in PROGMEM at x,y
//    the string is of the form "abc/def/ghi/jkl"
//    draws the nth substring
//-----------------------------------------------------------------------------
void DrawEnumerated(int x, int y, byte b, word aProgMem, word Font) {
  ST7735SetCursor(x, y);
  while (1) {
    switch (pgm_read_byte_near(aProgMem)) {
      case '/': if (b == 0) return; b--; break;
      case 0:  return;
      default: if (b == 0) DrawChar(pgm_read_byte_near(aProgMem), Font);
    }
    aProgMem++;
  }
}

//-----------------------------------------------------------------------------
//  SetMenuVariable
//    get or set a menu variable
//-----------------------------------------------------------------------------
void SetMenuVariable(char c) {
  switch (c) {
    case chMode:                  break;
    case chScale:                 SetScale(); break;
    case chLowestnote:            break;
    case chChord:                 SetChord(); break;
    case chPolyphony:             PlayNote(200, false); break;
    case chMaxDepth:              break;
    case chTremoloSize:           break;
    case chTremoloPeriod:         break;
    case chVibratoSize:           break;
    case chVibratoPeriod:         break;
    case chContinuousPitchRange:  break;
    case chContinuousMiddlenote:  break;
    case chInstrument:            SPItransferMidi(0xC0); SPItransferMidi(MenuVariable[chInstrument]); break;
    case chLeftHandKind:          SetLeftHandKind(); break;
    case chOctaves:               break;
    case chInversion:             SetChord(); break;
  }
}

//-----------------------------------------------------------------------------
//  MenuPlus
//    execute a menu "down" command
//-----------------------------------------------------------------------------
void MenuPlus() {
  if (IsMenuValue) {
    if (MenuCurVar != 255) {
      MenuVariable[MenuCurVar] = min(MenuVariable[MenuCurVar] + 1, MenuVariableMax[MenuCurVar]);
      SetMenuVariable(MenuCurVar);
      DrawMenu(true);
    }
  } else {
    MenuCur1++;
    DrawMenu(true);
  }
}

//-----------------------------------------------------------------------------
//  MenuMinus
//    execute a menu "up" command
//-----------------------------------------------------------------------------
void MenuMinus() {
  if (IsMenuValue) {
    if (MenuCurVar != 255) {
      MenuVariable[MenuCurVar] = max(MenuVariable[MenuCurVar] - 1, MenuVariableMin[MenuCurVar]);
      SetMenuVariable(MenuCurVar);
      DrawMenu(false);
    }
  } else {
    MenuCur1--;
    if (MenuCur1 < 0) {
      MenuCur0--;
      if (MenuCur0 < 0) {
        MenuCur0 = 0;
        MenuCur1 = 0; 
      } else {
        MenuCur1 = 10; // more than biggest numer of sub-menus
      }
      DrawMenu(false);
    } else {
      DrawMenu(false);
    }
  }
}

//-----------------------------------------------------------------------------
//  Keyboard drawing
//    note=255: draw previous key is up
//    note=254: draw whole keyboard
//    note=253: draw keyboard with scale marked
//    note=252: draw keyboard with chord marked
//    note:     draw note key is down
//-----------------------------------------------------------------------------
const int kbd_noteOffset[12] = {0, 10, 1, 11, 2, 3, 13, 4, 14, 5, 15, 6};
const int kbd_w = 9;
const int kbd_h = kbd_w * 3;
byte kbd_CentreNote = 60;
int kbd_y0;

int NotePos(int note) {
  int xoffset = -(kbd_CentreNote / 12 - 1) * 7 * kbd_w;
  int i = note % 12;
  i = kbd_noteOffset[i];
  if (i >= 10)
    return (i - 10) * kbd_w + (note / 12) * (kbd_w * 7) + kbd_w + xoffset; else
    return i * kbd_w + (note / 12) * (kbd_w * 7) + kbd_w / 2 + xoffset;
}

void DrawNoteKey(int note, bool whiteKey, uint16_t col) {
  ST7735CurColor = col;
  if (kbd_noteOffset[note % 12] >= 10)
    DrawBox(NotePos(note) - kbd_w / 3 - 1, kbd_y0, kbd_w * 2 / 3 + 1, kbd_y0 + kbd_h / 2 + 1 - (kbd_y0) - 1);
  else
    DrawBox(NotePos(note) - kbd_w / 2, kbd_y0, kbd_w - 1, kbd_y0 + kbd_h - (kbd_y0) - 1);
}

void DrawNoteSelected(int note, uint16_t col) { // if col == 0 then draw default color
  if (kbd_noteOffset[note % 12] >= 10) {
    if (col > 0)
      DrawNoteKey(note, false, col); else
      DrawNoteKey(note, false, ST7735_BLACK);
  } else {
    if (col > 0)
      DrawNoteKey(note, true, col); else
      DrawNoteKey(note, true, ST7735_WHITE);

    if ((note > 0) && (kbd_noteOffset[(note - 1) % 12] >= 10))
      DrawNoteKey(note - 1, false, ST7735_BLACK);
    if (kbd_noteOffset[(note + 1) % 12] >= 10)
      DrawNoteKey(note + 1, false, ST7735_BLACK);
  }
}

void DrawCnotes() {
  int i;
  ST7735CurColor = ST7735_BLACK;
  DrawBox(0, kbd_y0 + kbd_h + 2, 128, 8);
  ST7735CurColor = ST7735_WHITE;
  for (i = 0; i <= 127; i += 12) {
    DrawStringAt(NotePos(i) - 6, kbd_y0 + kbd_h + 8, "C", SmallFont);
    DrawInt(i / 12 - 1, SmallFont);
  }
}

void Marknotes(signed char s[], int note) {
  int i;
  for (i = 1; i <= s[0]; i++) {
    while ((NotePos(note+s[i]) < 0) && (kbd_CentreNote > 0))
      kbd_CentreNote -= 12;
    while ((NotePos(note+s[i]) > 120) && (kbd_CentreNote < 120))
      kbd_CentreNote += 12;
  }
  DrawNoteSelected(note+s[1], ST7735_RED);
  for (i = 2; i <= s[0]; i++) 
    DrawNoteSelected(note+s[i], ST7735_BLUE);
}

void DrawKeyboard2(int note) {
  int i;
  static int prevNote = 0;

  if (note == 255) { // draw previous key is up
    if (note != prevNote) {
      DrawNoteSelected(prevNote, 0);
      prevNote = note;
    }
  } else
  if (note < 128) { // draw note
    if (note != prevNote) {
      DrawNoteSelected(prevNote, 0);
      DrawNoteSelected(note, ST7735_RED);
      prevNote = note;
    }
  } else { // draw whole keyboard
    ST7735CurColor = ST7735_WHITE;
    for (i = 0; i <= 127; i++)
      if (kbd_noteOffset[i % 12] < 10)
        DrawNoteKey(i, true, ST7735_WHITE);
    for (i = 0; i <= 127; i++)
      if (kbd_noteOffset[i % 12] >= 10)
        DrawNoteKey(i, false, ST7735_BLACK);
        
    switch (note) {
      case 254: DrawCnotes(); break;
      case 253: Marknotes(Scale,MenuVariable[chLowestnote]); break;
      case 252: Marknotes(Chord,60); break;
    }      
  }
}

void DrawKeyboard(int note) {
  if (note < 128) {
    kbd_y0 = 16; 
    if ((NotePos(note) < 0) && (kbd_CentreNote > 0)) {
      DrawKeyboard2(255);
      while ((NotePos(note) < 0) && (kbd_CentreNote > 0))
        kbd_CentreNote -= 12;
      DrawCnotes();
    }
    if ((NotePos(note) > 120) && (kbd_CentreNote < 120)) {
      DrawKeyboard2(255);
      while ((NotePos(note) > 120) && (kbd_CentreNote < 120))
        kbd_CentreNote += 12;
      DrawCnotes();
    }
    DrawKeyboard2(note);
  } else 
  if (note >= 254) {
    kbd_y0 = 16; 
    DrawKeyboard2(note);
  } else {
    kbd_y0 = 0; 
    DrawKeyboard2(note);
  } 
}

//-----------------------------------------------------------------------------
//  DrawScale
//    draws the "piano" on the screen with the scale marked
//-----------------------------------------------------------------------------
void DrawScale() {
  ST7735CurColor = ST7735_BLACK;
  DrawBox(0,0,tft_width,kbd_h+2);
  DrawKeyboard(253);
}

//-----------------------------------------------------------------------------
//  DrawScale
//    draws the "piano" on the screen with the 
//-----------------------------------------------------------------------------
void DrawChord() {
  ST7735CurColor = ST7735_BLACK;
  DrawBox(0,0,tft_width,kbd_h+2);
  DrawKeyboard(252);
}

//-----------------------------------------------------------------------------
//  CheckMenuButton
//    if menu button down then "selects" menu
//-----------------------------------------------------------------------------
void CheckMenuButton(void) {
  byte i;

  if (digitalRead(BUTTON) == LOW) {
    Beep(255); 
    Beep(87); 

    delay(100); // for debounce
    for (i = 1; i<=100; i++) {
      if (digitalRead(BUTTON) == HIGH) {
        if (IsMenuing) {
          IsMenuValue = ! IsMenuValue;
          DrawMenu(true);
        } else {
          IncSetup();
          LoadFromEEPROM(CurSetupNumber); 
          DrawPlaying();
        }        
        delay(100); // for debounce
        return;
      }      
      delay(10);
    }
    // has been held down for 1 sec
    if (IsMenuing) {
      SaveToEEPROM(CurSetupNumber);
      IsMenuing = false;
      DrawPlaying();
    } else {
      IsMenuing = true;
      MenuCur0 = 0;
      MenuCur1 = 0;
      DrawMenu(true);
    }
    while (digitalRead(BUTTON) == LOW) ;
    delay(100); // for debounce
  }      
}

//-----------------------------------------------------------------------------
//  CheckFootPedal
//    if FootPedal down then increment the setup
//-----------------------------------------------------------------------------
void CheckFootPedal(void) {
  bool b;
  static bool prev = false;

  b = digitalRead(FOOT_PEDAL);
  if (b != prev)
    delay(100);

  if (IsMenuing && (b == LOW) && (prev != LOW)) {
    IncSetup();
    LoadFromEEPROM(CurSetupNumber); 
    DrawPlaying();
  }
      
  prev = b;
}

//-----------------------------------------------------------------------------
//  IncSetup
//    Increment setup number
//    load new setup
//-----------------------------------------------------------------------------
void IncSetup(void)
{
  CurSetupNumber = (CurSetupNumber % NumSetups) +1;
}

//-----------------------------------------------------------------------------
//  setup
//    run once at startup
//-----------------------------------------------------------------------------
void setup(void)
{
  int i;
  
  Serial.begin(57600);
  Serial.println("Midi Theremin");
  Serial.println("Build " __DATE__);    

  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);     delay(100);
  digitalWrite(LED_BUILTIN, LOW);      delay(100);
  digitalWrite(LED_BUILTIN, HIGH);     delay(100);
  digitalWrite(LED_BUILTIN, LOW);      delay(100);

  pinMode(MIDI_RST, OUTPUT);
  pinMode(MISO, OUTPUT);
  pinMode(MOSI, INPUT);
  pinMode(SCLK, OUTPUT);
  pinMode(MIDI_DREQ, INPUT);
  pinMode(MIDI_CS, OUTPUT);
  pinMode(UStrigR, OUTPUT);
  pinMode(USechoR, INPUT);
  pinMode(UStrigL, OUTPUT);
  pinMode(USechoL, INPUT);
  pinMode(BUTTON, INPUT_PULLUP);
  pinMode(FOOT_PEDAL, INPUT_PULLUP);
  digitalWrite(MIDI_CS, HIGH);
  digitalWrite(MIDI_RST, LOW);
  delay(10);
  digitalWrite(MIDI_RST, HIGH);

  SPI.begin ();
  SPI.setBitOrder(MSBFIRST);
  digitalWrite(LED_BUILTIN, HIGH);     delay(100);
  digitalWrite(LED_BUILTIN, LOW);      delay(100);
  digitalWrite(LED_BUILTIN, HIGH);     delay(100);
  digitalWrite(LED_BUILTIN, LOW);      delay(100);

  digitalWrite(UStrigR, LOW);
  digitalWrite(UStrigL, LOW);

  ST7735Begin(OLED_CS, OLED_DC, 128, 128);
  DrawStringAt(4,10, "MIDI Theremin", MediumBoldFont); 
  DrawStringAt(4,112, "Build " __DATE__, MediumFont); 
  delay(500);

  MenuVariable[chMode] = false; // discrete
  MenuVariable[chScale] = 0; // major Heptatonic
  MenuVariable[chOctaves] = 2; // 1..6
  MenuVariable[chLowestnote] = 60; // for discrete = middle C
  MenuVariable[chChord] = 0; // single note
  MenuVariable[chInversion] = 0; // no Inversion
  MenuVariable[chPolyphony] = 1; // no polyphony
  MenuVariable[chLeftHandKind] = 1; // Tremolo
  MenuVariable[chMaxDepth] = 10;          // 0..127
  MenuVariable[chTremoloSize] = 0;
  MenuVariable[chTremoloPeriod] = 20;
  MenuVariable[chVibratoSize] = 0;
  MenuVariable[chVibratoPeriod] = 20;
  MenuVariable[chContinuousPitchRange] = 12;
  MenuVariable[chContinuousMiddlenote] = 60; // middle C
  MenuVariable[chInstrument] = 0; // Grand Piano
  MenuVariable[chSwapHands] = 0; // hands normal

  if (EEPROM.read(0) != Version) // it's am empty EEPROM so initialise it
    for (i=1; i <= NumSetups; i++) 
      SaveToEEPROM(i);   
  
  LoadFromEEPROM(1);

  if (IsMenuing) {
    MenuCur0 = 0;
    MenuCur1 = 0;
    DrawMenu(true);
  } else {
    DrawPlaying();
  }

  PlayNote(255, false);
  SetBendRange(PitchWheelMax);
  SetScale();
  SPItransferMidi(0xC1); SPItransferMidi(45); // Instrument used for Beep

  Serial.println("Initialised");
}

//-----------------------------------------------------------------------------
// loop
//    main loop
//-----------------------------------------------------------------------------
void loop(void)
{
  long durationR, durationL;

  if (Serial.available()) {
    switch (Serial.read()) {
      case '+': if (IsMenuing) MenuPlus(); break;
      case '-': if (IsMenuing) MenuMinus(); break;
      case 'm': 
        IsMenuing = true;
        MenuCur0 = 0;
        MenuCur1 = 0;
        DrawMenu(true);
        break;
      case 'p': 
        IsMenuing = false;
        DrawPlaying();
        break;
      case 'v': 
        IsMenuValue = ! IsMenuValue;
        DrawMenu(true);
        break;
      case 0xFF: ExecSerialDebug();
    }
  }

  CheckMenuButton();
  CheckFootPedal();

  if (MenuVariable[chSwapHands]) {
    durationL = GetSonar(UStrigR, USechoR); // swapped
    durationR = GetSonar(UStrigL, USechoL);
  } else {
    durationR = GetSonar(UStrigR, USechoR); // hands normal
    durationL = GetSonar(UStrigL, USechoL);
  }

  ExecLeftHand(durationL);

  ApplyTremolo();

  if (MenuVariable[chMode]) {
    ExecContinuous(durationR);
  } else {
    ExecDiscrete(durationR);
  }
}
